@@ -8,8 +8,10 @@ from prompt_toolkit.history import FileHistory
 from prompt_toolkit.lexers import PygmentsLexer
 
 from .Commands import *
+from .Completer import *
 from .Highlighter import *
 from .Expressions import parseExpression, evaluateExpression
+from .Indenter import bindings
 from .Utils.ColoredLogs import ColorizedArgsFormatter
 
 class Shell:
@@ -20,6 +22,7 @@ class Shell:
         self.builtin_variables = {}
         self.variables = {}
         self.commands = {}
+        self.completer = Completer()
         self.addCommand('exit', Exit)
         self.addCommand('help', Help)
         self.addCommand('echo', Echo)
@@ -64,12 +67,14 @@ class Shell:
             exit(8)
         self.commands[name] = cls(self)
         ShellLexer.addCommand(name)
+        self.completer.addCommand(name, cls)
 
     def addBuiltinVariable(self, name, value):
         if name in self.builtin_variables:
             logger.critical('A builtin variable with the same name exists')
             exit(11)
         self.builtin_variables[name] = value
+        self.completer.addVariable(name)
 
     def runScript(self, script, shell_after=True):
         # If script specified, open its file
@@ -97,13 +102,14 @@ class Shell:
             else:
                 final_prompt.append((final_prompt[-1][0], ' '))
             # Start the prompt, add styles in the same manner as prompt_toolkit
-            user_command = prompt(final_prompt, style=self.style, history=FileHistory(self.history), lexer=PygmentsLexer(ShellLexer))
+            user_command = prompt(final_prompt, style=self.style, history=FileHistory(self.history), lexer=PygmentsLexer(ShellLexer), key_bindings=bindings, completer=self.completer.getCompleter())
             self.runCommand(user_command)
 
     def runCommand(self, entire_command):
         # Ignore empty lines and comments
         if entire_command == '' or entire_command[0] == '#':
             return
+        entire_command = entire_command.strip()
         # Find the called command first
         user_command = entire_command.split(' ', 1)
         command = user_command[0]
